@@ -62,6 +62,11 @@ def set_rnr_attributes(qp_attr):
     qp_attr.rnr_retry = RNR_RETRY
 
 
+def is_gid_available(gid_index):
+    if gid_index is None:
+        raise unittest.SkipTest(f'No relevant GID found')
+
+
 class PyverbsAPITestCase(unittest.TestCase):
     def __init__(self, methodName='runTest'):
         super().__init__(methodName)
@@ -71,6 +76,7 @@ class PyverbsAPITestCase(unittest.TestCase):
         self.ctx = None
         self.attr = None
         self.attr_ex = None
+        self.gid_index = 0
 
     def setUp(self):
         """
@@ -88,6 +94,9 @@ class PyverbsAPITestCase(unittest.TestCase):
                 raise unittest.SkipTest('No IB devices found')
             self.dev_name = dev_list[0].name.decode()
 
+        if self.config['gid']:
+            self.gid_index = self.config['gid']
+
         self.create_context()
         self.attr = self.ctx.query_device()
         self.attr_ex = self.ctx.query_device_ex()
@@ -222,6 +231,9 @@ class RDMATestCase(unittest.TestCase):
         port_attrs = ctx.query_port(port)
         if port_attrs.state != e.IBV_PORT_ACTIVE:
             return
+        if not port_attrs.gid_tbl_len:
+            self._get_ip_mac(dev, port, None)
+            return
         dev_attrs = ctx.query_device()
         vendor_id = dev_attrs.vendor_id
         vendor_pid = dev_attrs.vendor_part_id
@@ -311,6 +323,7 @@ class RDMACMBaseTest(RDMATestCase):
         if not self.ip_addr:
             raise unittest.SkipTest('Device {} doesn\'t have net interface'
                                     .format(self.dev_name))
+        is_gid_available(self.gid_index)
 
     def two_nodes_rdmacm_traffic(self, connection_resources, test_flow,
                                  **resource_kwargs):
@@ -362,6 +375,9 @@ class RDMACMBaseTest(RDMATestCase):
             res, side = self.notifier.get()
             proc_res[side] = res
         for ex in proc_res.values():
+            if isinstance(ex, PyverbsRDMAError) and \
+                    ex.error_code == errno.EOPNOTSUPP:
+                        raise unittest.SkipTest(ex)
             if isinstance(ex, unittest.case.SkipTest):
                 raise(ex)
         if proc_res:
@@ -522,9 +538,9 @@ class TrafficResources(BaseResources):
         :return: None
         """
         self.port_attr = self.ctx.query_port(self.ib_port)
+        self.create_cq()
         if self.with_srq:
             self.create_srq()
-        self.create_cq()
         self.create_mr()
         self.create_qps()
 
@@ -599,7 +615,13 @@ class TrafficResources(BaseResources):
         raise NotImplementedError()
 
 
-class RCResources(TrafficResources):
+class RoCETrafficResources(TrafficResources):
+    def __init__(self, dev_name, ib_port, gid_index, **kwargs):
+        is_gid_available(gid_index)
+        super(RoCETrafficResources, self).__init__(dev_name, ib_port, gid_index, **kwargs)
+
+
+class RCResources(RoCETrafficResources):
 
     def to_rts(self):
         """
@@ -635,7 +657,7 @@ class RCResources(TrafficResources):
         self.to_rts()
 
 
-class UDResources(TrafficResources):
+class UDResources(RoCETrafficResources):
     UD_QKEY = 0x11111111
     UD_PKEY_INDEX = 0
     GRH_SIZE = 40
@@ -654,10 +676,15 @@ class UDResources(TrafficResources):
         qp_attr.qkey = self.UD_QKEY
         qp_attr.pkey_index = self.UD_PKEY_INDEX
         for _ in range(self.qp_count):
-            qp = QP(self.pd, qp_init_attr, qp_attr)
-            self.qps.append(qp)
-            self.qps_num.append(qp.qp_num)
-            self.psns.append(random.getrandbits(24))
+            try:
+                qp = QP(self.pd, qp_init_attr, qp_attr)
+                self.qps.append(qp)
+                self.qps_num.append(qp.qp_num)
+                self.psns.append(random.getrandbits(24))
+            except PyverbsRDMAError as ex:
+                if ex.error_code == errno.EOPNOTSUPP:
+                    raise unittest.SkipTest(f'Create QP type {qp_init_attr.qp_type} is not supported')
+                raise ex
 
     def pre_run(self, rpsns, rqps_num):
         self.rpsns = rpsns
@@ -670,7 +697,7 @@ class RawResources(TrafficResources):
                           rcq=self.cq, srq=self.srq, cap=self.create_qp_cap())
 
 
-class XRCResources(TrafficResources):
+class XRCResources(RoCETrafficResources):
     def __init__(self, dev_name, ib_port, gid_index, qp_count=2):
         self.temp_file = None
         self.xrcd_fd = -1
