@@ -13,14 +13,16 @@ import socket
 import struct
 import string
 import glob
+import time
 import os
 
 from pyverbs.pyverbs_error import PyverbsError, PyverbsRDMAError
 from pyverbs.addr import AHAttr, AH, GlobalRoute
-from tests.base import XRCResources, DCT_KEY
+from tests.base import XRCResources, DCT_KEY, MLNX_VENDOR_ID
 from tests.efa_base import SRDResources
+from pyverbs.providers.efa.efadv import EfaCQ
 from pyverbs.wr import SGE, SendWR, RecvWR
-from pyverbs.qp import QPCap, QPInitAttr, QPInitAttrEx
+from pyverbs.qp import QPCap, QPInitAttr, QPInitAttrEx, QPAttr, QPEx, QP
 from tests.mlx5_base import Mlx5DcResources, Mlx5DcStreamsRes
 from pyverbs.base import PyverbsRDMAErrno
 from pyverbs.cq import PollCqAttr, CQEX
@@ -44,6 +46,7 @@ MAX_DM_LOG_ALIGN = 6
 MAX_RAW_PACKET_SEND_WR = 2500
 GRH_SIZE = 40
 IMM_DATA = 1234
+POLL_CQ_TIMEOUT = 10  # In seconds
 
 
 class MatchCriteriaEnable:
@@ -269,13 +272,17 @@ def get_create_qp_flags_raw_packet(attr_ex):
     return val
 
 
-def random_qp_create_flags(qpt, attr_ex):
+def random_valid_qp_create_flags(qpt, attr, attr_ex):
     """
     Select a random sublist of ibv_qp_create_flags according to the QP type.
     :param qpt: Current QP type
     :param attr_ex: Used for Raw Packet QP to check device capabilities
     :return: A sublist of ibv_qp_create_flags
     """
+     # Most HCAs doesn't support any create_flags so far except mlx4/mlx5
+    if attr.vendor_id != MLNX_VENDOR_ID:
+        return 0
+
     if qpt == e.IBV_QPT_RAW_PACKET:
         return get_create_qp_flags_raw_packet(attr_ex)
     elif qpt == e.IBV_QPT_UD:
@@ -307,7 +314,7 @@ def random_qp_init_attr_ex(attr_ex, attr, qpt=None):
     sig = random.randint(0, 1)
     mask = random_qp_create_mask(qpt, attr_ex)
     if mask & e.IBV_QP_INIT_ATTR_CREATE_FLAGS:
-        cflags = random_qp_create_flags(qpt, attr_ex)
+        cflags = random_valid_qp_create_flags(qpt, attr, attr_ex)
     else:
         cflags = 0
     if mask & e.IBV_QP_INIT_ATTR_MAX_TSO_HEADER:
@@ -403,7 +410,7 @@ def get_recv_wr(agr_obj):
     :return: recv wr
     """
     qp_type = agr_obj.rqp_lst[0].qp_type if isinstance(agr_obj, XRCResources) \
-                else agr_obj.qp.qp_type
+        else agr_obj.qp.qp_type if isinstance(agr_obj.qp, QP) else None
     mr = agr_obj.mr
     length = agr_obj.msg_size + GRH_SIZE if qp_type == e.IBV_QPT_UD \
              else agr_obj.msg_size
@@ -524,7 +531,49 @@ def post_recv(agr_obj, recv_wr, qp_idx=0 ,num_wqes=1):
     """
     receive_queue = agr_obj.srq if agr_obj.srq else agr_obj.qps[qp_idx]
     for _ in range(num_wqes):
-        receive_queue.post_recv(recv_wr, None)
+        if isinstance(receive_queue, QPEx) and receive_queue.ind_table:
+            for wq in receive_queue.ind_table.wqs:
+                wq.post_recv(recv_wr, None)
+        else:
+            receive_queue.post_recv(recv_wr, None)
+
+
+def _poll_cq(cq, count=1, data=None):
+    """
+    Poll <count> completions from the CQ.
+    Note: This function calls the blocking poll() method of the CQ
+    until <count> completions were received. Alternatively, gets a
+    single CQ event when events are used.
+    :param cq: CQ to poll from
+    :param count: How many completions to poll
+    :param data: In case of a work request with immediate, the immediate data
+                 to be compared after poll
+    :return: An array of work completions of length <count>, None
+             when events are used
+    """
+    wcs = []
+    channel = cq.comp_channel
+    start_poll_t = time.perf_counter()
+    while count > 0 and (time.perf_counter() - start_poll_t < POLL_CQ_TIMEOUT):
+        if channel:
+            channel.get_cq_event(cq)
+            cq.req_notify()
+        nc, tmp_wcs = cq.poll(count)
+        for wc in tmp_wcs:
+            if wc.status != e.IBV_WC_SUCCESS:
+                wcs.append(wc)
+                return wcs
+            if data:
+                if wc.wc_flags & e.IBV_WC_WITH_IMM == 0:
+                    raise PyverbsRDMAError('Completion without immediate')
+                assert socket.ntohl(wc.imm_data) == data
+        count -= nc
+        wcs.extend(tmp_wcs)
+
+    if count > 0:
+        raise PyverbsError(f'Got timeout on polling ({count} CQEs remaining)')
+
+    return wcs
 
 
 def poll_cq(cq, count=1, data=None):
@@ -540,40 +589,29 @@ def poll_cq(cq, count=1, data=None):
     :return: An array of work completions of length <count>, None
              when events are used
     """
-    wcs = []
-    channel = cq.comp_channel
-    while count > 0:
-        if channel:
-            channel.get_cq_event(cq)
-            cq.req_notify()
-        nc, tmp_wcs = cq.poll(count)
-        for wc in tmp_wcs:
-            if wc.status != e.IBV_WC_SUCCESS:
-                raise PyverbsRDMAError('Completion status is {s}'.
-                                       format(s=wc_status_to_str(wc.status)),
-                                       wc.status)
-            if data:
-                if wc.wc_flags & e.IBV_WC_WITH_IMM == 0:
-                    raise PyverbsRDMAError('Completion without immediate')
-                assert socket.ntohl(wc.imm_data) == data
-        count -= nc
-        wcs.extend(tmp_wcs)
+    wcs = _poll_cq(cq, count, data)
+    if wcs[0].status != e.IBV_WC_SUCCESS:
+        raise PyverbsRDMAError(f'Completion status is {wc_status_to_str(wcs[0].status)}')
+
     return wcs
 
 
-def poll_cq_ex(cqex, count=1, data=None):
+def poll_cq_ex(cqex, count=1, data=None, sgid=None):
     """
     Poll <count> completions from the extended CQ.
     :param cq: CQEX to poll from
     :param count: How many completions to poll
     :param data: In case of a work request with immediate, the immediate data
                  to be compared after poll
+    :param sgid: In case of EFA receive completion, the sgid to be compared
+                 after poll
     :return: None
     """
     try:
+        start_poll_t = time.perf_counter()
         poll_attr = PollCqAttr()
         ret = cqex.start_poll(poll_attr)
-        while ret == 2: # ENOENT
+        while ret == 2 and (time.perf_counter() - start_poll_t < POLL_CQ_TIMEOUT):
             ret = cqex.start_poll(poll_attr)
         if ret != 0:
             raise PyverbsRDMAErrno('Failed to poll CQ')
@@ -583,19 +621,30 @@ def poll_cq_ex(cqex, count=1, data=None):
                                    format(s=cqex.status))
         if data:
             assert data == socket.ntohl(cqex.read_imm_data())
+
+        if isinstance(cqex, EfaCQ):
+            if sgid is not None and cqex.read_opcode() == e.IBV_WC_RECV:
+                assert sgid.gid == cqex.read_sgid().gid
         # Now poll the rest of the packets
-        while count > 0:
+        while count > 0 and (time.perf_counter() - start_poll_t < POLL_CQ_TIMEOUT):
             ret = cqex.poll_next()
             while ret == 2:
                 ret = cqex.poll_next()
             if ret != 0:
                 raise PyverbsRDMAErrno('Failed to poll CQ')
+            count -= 1
             if cqex.status != e.IBV_WC_SUCCESS:
                 raise PyverbsRDMAErrno('Completion status is {s}'.
                                        format(s=cqex.status))
             if data:
                 assert data == socket.ntohl(cqex.read_imm_data())
+
+            if isinstance(cqex, EfaCQ):
+                if sgid is not None and cqex.read_opcode() == e.IBV_WC_RECV:
+                    assert sgid.gid == cqex.read_sgid().gid
             count -= 1
+        if count > 0:
+            raise PyverbsError(f'Got timeout on polling ({count} CQEs remaining)')
     finally:
         cqex.end_poll()
 
@@ -749,6 +798,8 @@ def gen_packet(msg_size, l3=PacketConsts.IP_V4, l4=PacketConsts.UDP_PROTO, with_
     :param kwargs: Arguments:
             * *src_mac*
                 Source MAC address to use in the packet.
+            * *src_ipv4*
+                Source IPv4 address to use in the packet.
     :return: packet
     """
     l3_header_size = getattr(PacketConsts, f'IPV{str(l3)}_HEADER_SIZE')
@@ -775,11 +826,12 @@ def gen_packet(msg_size, l3=PacketConsts.IP_V4, l4=PacketConsts.UDP_PROTO, with_
 
     if l3 == PacketConsts.IP_V4:
         # IPv4 header
+        src_ipv4 = kwargs.get('src_ipv4', PacketConsts.SRC_IP)
         packet += struct.pack('!2B3H2BH4s4s', (PacketConsts.IP_V4 << 4) +
                               PacketConsts.IHL, 0, ip_total_len, 0,
                               PacketConsts.IP_V4_FLAGS << 13,
                               PacketConsts.TTL_HOP_LIMIT, next_hdr, 0,
-                              socket.inet_aton(PacketConsts.SRC_IP),
+                              socket.inet_aton(src_ipv4),
                               socket.inet_aton(PacketConsts.DST_IP))
     else:
         # IPv6 header
@@ -868,6 +920,51 @@ def raw_traffic(client, server, iters, l3=PacketConsts.IP_V4,
                          expected_packet if expected_packet else msg, skip_idxs)
 
 
+def raw_rss_traffic(client, server, iters, l3=PacketConsts.IP_V4,
+                    l4=PacketConsts.UDP_PROTO, with_vlan=False, num_packets=1):
+    """
+    Runs raw ethernet rss traffic between two sides.
+    :param client: client side, clients base class is BaseTraffic
+    :param server: server side, servers base class is BaseTraffic
+    :param iters: number of traffic iterations
+    :param l3: Packet layer 3 type: 4 for IPv4 or 6 for IPv6
+    :param l4: Packet layer 4 type: 'tcp' or 'udp'
+    :param with_vlan: if True add VLAN header to the packet
+    :param num_packets: Number of packets to send with different ipv4 src
+                        address in each iteration.
+    :return: None
+    """
+    s_recv_wr = get_recv_wr(server)
+    for qp_idx in range(server.qp_count):
+        # prepare the receive queue with RecvWR
+        post_recv(server, s_recv_wr, qp_idx=qp_idx, num_wqes=num_packets)
+    for _ in range(iters):
+        for qp_idx in range(server.qp_count):
+            for i in range(num_packets):
+                c_send_wr, c_sg, msg = get_send_elements_raw_qp(
+                    client, l3, l4, with_vlan,
+                    src_ipv4='.'.join([str(num) for num in range(i, i + 4)]))
+                send(client, c_send_wr, e.IBV_WR_SEND, False, qp_idx)
+                poll_cq(client.cq)
+            completions = 0
+            start_poll_t = time.perf_counter()
+            while completions < num_packets and \
+                    (time.perf_counter() - start_poll_t < POLL_CQ_TIMEOUT):
+                for cq in server.cqs:
+                    n, wcs = cq.poll()
+                    if n > 0:
+                        if wcs[0].status != e.IBV_WC_SUCCESS:
+                            raise PyverbsRDMAError(
+                                f'Completion status is {wc_status_to_str(wcs[0].status)}',
+                                wcs[0].status)
+                        completions += 1
+                        if completions >= num_packets:
+                            break
+            if completions < num_packets:
+                raise PyverbsError(f'Expected {num_packets} completions - got {completions}')
+            post_recv(server, s_recv_wr, qp_idx=qp_idx, num_wqes=num_packets)
+
+
 def rdma_traffic(client, server, iters, gid_idx, port, new_send=False,
                  send_op=None):
     """
@@ -1227,3 +1324,82 @@ def is_datagram_qp(agr_obj):
 
 def is_root():
     return os.geteuid() == 0
+
+
+def post_rq_state_bad_flow(test_obj):
+    """
+    Check post_recive on rq while qp is in invalid state.
+    - Change qp's state to IBV_QPS_RESET
+    - Verify post receive on qp fails
+    :param test_obj: An instance of RDMATestCase
+    :return: None.
+    """
+    qp_attr = QPAttr(qp_state=e.IBV_QPS_RESET, cur_qp_state=e.IBV_QPS_RTS)
+    test_obj.server.qps[0].modify(qp_attr, e.IBV_QP_STATE)
+    recv_wr = get_recv_wr(test_obj.server)
+    with test_obj.assertRaises(PyverbsRDMAError) as ex:
+        post_recv(test_obj.server, recv_wr, qp_idx=0)
+    test_obj.assertEqual(ex.exception.error_code, errno.EINVAL)
+
+
+def post_sq_state_bad_flow(test_obj):
+    """
+    Check post_send on sq while qp is in invalid state.
+    - Change qp's state to IBV_QPS_RESET
+    - Verify post send on qp fails
+    :param test_obj: An instance of RDMATestCase
+    :return: None.
+    """
+    qp_idx = 0
+    qp_attr = QPAttr(qp_state=e.IBV_QPS_RESET, cur_qp_state=e.IBV_QPS_RTS)
+    test_obj.client.qps[qp_idx].modify(qp_attr, e.IBV_QP_STATE)
+    ah = get_global_ah(test_obj.client, test_obj.gid_index, test_obj.ib_port)
+    _, sg = get_send_elements(test_obj.client, False)
+    with test_obj.assertRaises(PyverbsRDMAError) as ex:
+        send(test_obj.client, sg, e.IBV_QP_EX_WITH_SEND, new_send=True,
+             qp_idx=qp_idx, ah=ah)
+    test_obj.assertEqual(ex.exception.error_code, errno.EINVAL)
+
+
+def full_rq_bad_flow(test_obj):
+    """
+    Check post_recive while qp's rq is full.
+    - Find qp's rq length.
+    - Fill the qp with work requests until overflow.
+    :param test_obj: An instance of RDMATestCase
+    :return: None.
+    """
+    qp_attr, _ = test_obj.server.qps[0].query(e.IBV_QP_CAP)
+    max_recv_wr = qp_attr.cap.max_recv_wr
+    with test_obj.assertRaises(PyverbsRDMAError) as ex:
+        for _ in range (max_recv_wr + 1):
+            s_recv_wr = get_recv_wr(test_obj.server)
+            post_recv(test_obj.server, s_recv_wr, qp_idx=0)
+    test_obj.assertEqual(ex.exception.error_code, errno.ENOMEM)
+
+
+def create_rq_with_larger_sgl_bad_flow(test_obj):
+    """
+    Check post_receive on qp while wr sgl is bigger than
+    max sge allowed for the qp
+    - Find max sge allowed for the qp
+    - Create wr with sgl bigger than the max
+    - Verify post receive on qp fails
+    :param test_obj: An instance of RDMATestCase
+    :return: None.
+    """
+    qp_idx = 0
+    server_mr = test_obj.server.mr
+    server_mr_buf = server_mr.buf
+    qp_attr, _ = test_obj.server.qps[qp_idx].query(e.IBV_QP_CAP)
+    max_recv_sge = qp_attr.cap.max_recv_sge
+    length = test_obj.server.msg_size // (max_recv_sge + 1)
+    sgl = []
+    offset = 0
+    for _ in range(max_recv_sge + 1):
+        sgl.append(SGE(server_mr_buf + offset, length, server_mr.lkey))
+        offset = offset + length
+    s_recv_wr = RecvWR(sg=sgl, num_sge=max_recv_sge + 1)
+    with test_obj.assertRaises(PyverbsRDMAError) as ex:
+        post_recv(test_obj.server, s_recv_wr, qp_idx=qp_idx)
+    test_obj.assertEqual(ex.exception.error_code, errno.EINVAL)
